package golfCourse.agents.managerAgent;

import golfCourse.Tools;
import golfCourse.agents.GolfCourseAgent;
import golfCourse.agents.contactToOTASystemAgent.ContactToOTASystemAgent;
import golfCourse.agents.databaseAgent.DataBaseAgent;
import golfCourse.agents.managerAgent.behaviours.ReceiveSystemErrorBehaviour;
import golfCourse.agents.managerAgent.behaviours.StartSystemBehaviour;
import golfCourse.agents.managerAgent.customer.behaviours.CloseSystemBehaviour;
import golfCourse.agents.searchAgent.SearchAgent;
import golfCourse.agents.translationAgent.TranslationAgent;
import golfCourse.communication.ontology.GolfCourseOntology;
import golfCourse.communication.services.ServiceFinder;
import golfCourse.communication.services.ServiceFinderException;
import golfCourse.communication.services.ServiceWaiter;
import golfCourse.properties.GolfCourseSystemProperties;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.ParallelBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import java.io.IOException;
import java.util.Properties;

/**
 * Main agent in system.
 * This agent communicates with customers, start and close system.
 */
public class ManagerAgent extends GolfCourseAgent{

	public final static String SERVICE_NAME = "Manager Services";
	public final static String SERVICE_TYPE = "manager-service";
	public final static String SERVICE_OWNERSHIP = "Golf System";
	
	public ServiceFinder translationService;
	public ServiceFinder searchingServices;
	public ServiceFinder databaseServices;
	public ServiceFinder contactToExternalOTASystemServices;
	
	//	 Keys used in property files.
	public final static String COUNT_TA = "TA";
	public final static String COUNT_SA = "SA";

	private int countTA = 1;
	private int countQA = 1;
	private final int countDBA = 1;
	
	private static String LOAD_DATA = "LOAD_DATA";
	private static String REMOVE_DATA = "REMOVE_DATA";
	private static String CLOSE_SYSTEM = "CLOSE_SYSTEM";
	private static String USER_CONVERSATION_ID = "USER_CONVERSATION_ID";
	private static String CA_CONVERSATION_ID = "CA_CONVERSATION_ID";
	
	public String customerAnswer = "2";
	public String loadDataMessage = "";
	public String removeDataMessage = "";
	public String closeSystemMessage = "";
	public String conversationID = "1";

	
	private static Logger logger = Logger.getMyLogger(ManagerAgent.class.getName());
			
	public ManagerAgent()
	{
		super();
	}
	
	protected void setup() {
		
		findServices();
		
		readAgentProperties();
		
		readMessagesProperties();
		
		addBehaviour(new SequentialBehaviour(this) {
			public void onStart() {	
				
				// Create agents in system
				addSubBehaviour(new StartSystemBehaviour(countTA, countQA));
				
				// Add behaviour waiting for registration of services the agent depends on
				addSubBehaviour(new ParallelBehaviour(myAgent, ParallelBehaviour.WHEN_ALL) {
								
					public void onStart() {
						// wait for translation agents
						//addSubBehaviour(new AllServicesWaiter(myAgent, translationService, countTA));
						//addSubBehaviour(new AllServicesWaiter(myAgent, searchingServices, countQA));
						addSubBehaviour(new ServiceWaiter(myAgent, translationService));
						addSubBehaviour(new ServiceWaiter(myAgent, searchingServices));
						addSubBehaviour(new ServiceWaiter(myAgent, databaseServices/*, countDBA*/));

					}
				});
	
				// add main behaviour - receiving messages
				addSubBehaviour(new OneShotBehaviour(myAgent) {
					public void action() {
						
						// Initialize responding functionality.
						MessageTemplate mt = MessageTemplate
								.MatchOntology(GolfCourseOntology.ONTOLOGY_NAME);
						
						addBehaviour(new ManagerAgentSystemResponder(myAgent, mt));
						
						addBehaviour(new ManagerAgentCustomerBehaviour((ManagerAgent)myAgent));

						//addBehaviour(new ManagerAgentBehaviour());
					}
				});
			}
		});	
		
		registerInDF();
		logger.info(getLocalName() + " is created");
	}
	
	private void findServices()
	{
		try {
			translationService = new ServiceFinder(this, TranslationAgent.getServiceDescription());
		} catch(ServiceFinderException e)
		{
			logger.info("Manager Agent: no translation agents were found.");
		}
		
		try {
			searchingServices = new ServiceFinder(this, SearchAgent.getServiceDescription());
		} catch(ServiceFinderException e)
		{
			logger.info("Manager Agent: no search agents were found.");
		}
		
		try {
			databaseServices = new ServiceFinder(this, DataBaseAgent.getServiceDescription());
		} catch(ServiceFinderException e)
		{
			logger.info("Manager Agent: no database agent were found.");
		}
		
		try {
			this.contactToExternalOTASystemServices = new ServiceFinder(this, ContactToOTASystemAgent.getServiceDescription());
		}
		catch(ServiceFinderException e)
		{
			logger.info("Manager Agent: no contact agents were found");
		}
	}
	
	protected void registerInDF() {
		try {
			ServiceDescription sd = getServiceDescription();			
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			dfd.addServices(sd);
			DFService.register(this, dfd);
			logger.info(getLocalName() + " is register");
		} catch (FIPAException e) {
			logger.info(getLocalName() + " - cannot register translation agent at DF: " + e.getLocalizedMessage());
			doDelete();
		}
	}
	/**
	 * Returns information about the service this agent provides
	 * 
	 * @return service description
	 */
	public static ServiceDescription getServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setType(SERVICE_TYPE);
		sd.setName(SERVICE_NAME);
		sd.setOwnership(SERVICE_OWNERSHIP);
		return sd;
	}

	/**
	 * Read user properties: count of TranslationAgents and SearchingAgents.
	 */
	private void readAgentProperties()
	{
		try{
			logger.info("Agent properties: " + GolfCourseSystemProperties.AGENTS_PROPERTIES);
			Properties prop = Tools.loadProperties(GolfCourseSystemProperties.AGENTS_PROPERTIES);
			String ta = prop.getProperty(COUNT_TA);
			String sa = prop.getProperty(COUNT_SA);
			
			countTA = Integer.parseInt(ta);
			countQA = Integer.parseInt(sa);
		}
		catch(IOException e)
		{
			// It's only warning - one SA and one TA will be created.
			logger.info("Manager Agent: cannot read agents-properties file - " + e.getLocalizedMessage());
			logger.info("Manager Agent cannot read agents-properties file - " + e.getLocalizedMessage());
		}
	}
	
	private void readMessagesProperties()
	{
		try{
			Properties prop = Tools.loadProperties(GolfCourseSystemProperties.USER_MESSAGES);
			this.loadDataMessage = prop.getProperty(LOAD_DATA);
			this.conversationID = prop.getProperty(USER_CONVERSATION_ID);
			this.removeDataMessage = prop.getProperty(REMOVE_DATA);
			this.closeSystemMessage = prop.getProperty(CLOSE_SYSTEM);
			this.customerAnswer = prop.getProperty(CA_CONVERSATION_ID);
		}
		catch(IOException e)
		{	
			logger.info("Manager Agent: cannot read user messages file - " + e.getMessage());
			addBehaviour(new ReceiveSystemErrorBehaviour("Manager Agent: cannot read user messages file - " + e.getMessage(), this));
		}
	}
	
	protected void takeDown() 
    {
		// add sending close meesage for another agents
		addBehaviour(new CloseSystemBehaviour(this));

		try {
			unregisterInDF();
			logger.info(getLocalName() + " is unregistered");
		}
		catch(FIPAException e)
		{
			logger.info(getLocalName() + " cannot be unregistered: " + e.getLocalizedMessage());
		}
    }
	
	protected void getManagerAID()
	{
		this.managerAID = getAID();
	}
}
