package golfCourse.agents.searchAgent;

import golfCourse.agents.GolfCourseAgent;
import golfCourse.agents.SendSystemErrorBehaviour;
import golfCourse.agents.translationAgent.TranslationAgent;
import golfCourse.communication.ontology.GolfCourseOntology;
import golfCourse.communication.services.ServiceFinder;
import golfCourse.communication.services.ServiceFinderException;
import golfCourse.ontologies.jastor.golfCourse.GolfCourse;
import golfCourse.ontology.storage.ModelLoader;
import golfCourse.ontology.storage.ModelLoaderInitException;
import golfCourse.properties.GolfCourseSystemProperties;
import jade.core.AID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import java.util.Properties;

import com.hp.hpl.jena.ontology.OntModel;

public class SearchAgent extends GolfCourseAgent{

	private static Logger logger = Logger.getMyLogger(SearchAgent.class.getName());
	
	public final static String SERVICE_NAME = "Golf Course service";
	public final static String SERVICE_TYPE = "golfCourse-service";
	public final static String SERVICE_OWNERSHIP = "Golf Course System";

	private OntModel mDomain;
    private OntModel mGolfCoursesDB;
    
    public ServiceFinder translationService;

	//*****************************************
	// Provides models for behaviours
	//*****************************************    
	/**
	 * Returns ontology model describing domain of restaurant world.
	 * @return
	 */
	public OntModel getDomainModel()				
	{ 
		return mDomain; 
	}
	/**
	 * Returns database with instances of restaurants.
	 * @return
	 */
	public OntModel getGolfCoursesDB()				
	{ 
		return mGolfCoursesDB; 
	}

	/**
	 * Constructs RestaurantServiceAgent.
	 */
	public SearchAgent() {
		super();
		
		// Read necessary ontology models.
		loadOntologyModels();
	}
	
	private void findServices()
	{
		try {
			translationService = new ServiceFinder(this, TranslationAgent.getServiceDescription());
		} catch(ServiceFinderException e)
		{
			logger.info(getLocalName() + " no translations agents were found.");
		}
	}
	
	private void loadOntologyModels() {
		
		logger.info("Search agent: loading golf course ontology ... ");
				
		mDomain = ModelLoader.loadModel(GolfCourseSystemProperties.DOMAIN_GOLF_COURSE);
		logger.info("Search agent: domain model is loaded");
				
		logger.info("Search agent: loading golf courses db from DB ...");
		try{
			mGolfCoursesDB = ModelLoader.loadDBModel(GolfCourse.TYPE.getNameSpace(), GolfCourseSystemProperties.DB_GOLF_COURSE);
			logger.info("Search agent: databse model is loaded");
		} catch (ModelLoaderInitException e) {
			logger.info("Search agent - problem while initializing ModelLoader." + e.getLocalizedMessage());
			
			this.addBehaviour(new SendSystemErrorBehaviour(getLocalName() + " - problem while initializing ModelLoader." + e.getMessage(), this));
		}
		}
	
	protected void activateProperties(Properties conf) {		
	}	

	protected void setup() {
		
		// Initialize responding functionality.
		MessageTemplate mt = MessageTemplate
				.MatchOntology(GolfCourseOntology.ONTOLOGY_NAME);
		
		addBehaviour(new SearchAgentResponder(this, mt));

		// Register in DF>		
		registerInDF();
		
		logger.info(getLocalName() + " is created");
		
		findServices();
	}

	protected void registerInDF()
	{
		try {
			ServiceDescription sd = getServiceDescription();			
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			dfd.addServices(sd);
			DFService.register(this, dfd);
			logger.info(getLocalName() + " is register");
		} catch (FIPAException e) {
			logger.info(getLocalName() + " cannot register agent at DF: " + e.getMessage());
			doDelete();
		}
	}
	/**
	 * Returns information about the service this agent provides
	 * 
	 * @return service description
	 */
	public static ServiceDescription getServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setType(SERVICE_TYPE);
		sd.setName(SERVICE_NAME);
		sd.setOwnership(SERVICE_OWNERSHIP);
		return sd;
	}
	
	protected void getManagerAID()
	{
		Object[] args = getArguments();
		if(args!= null)
		{
			if(args.length > 0)
			{
				this.managerAID = (AID)args[0];
				return;
			}
		}
	}
	
	protected void takeDown() 
    {
		try {
			unregisterInDF();
			logger.info(getLocalName() + " is unregistered");
		}
		catch(FIPAException e)
		{
			logger.info(getLocalName() + " cannot be unregistered: " + e.getMessage());
		}
    }
}
