package golfCourse.agents.searchAgent.handlers;

import golfCourse.agents.SendErrorToExternalBehaviour;
import golfCourse.agents.SendErrorToOTABehaviour;
import golfCourse.agents.SendSystemErrorBehaviour;
import golfCourse.agents.searchAgent.SearchAgent;
import golfCourse.communication.ontology.Conditions;
import golfCourse.communication.ontology.MA_send_ontologies;
import golfCourse.communication.ontology.OWLData;
import golfCourse.communication.ontology.SA_return_golf_courses;
import golfCourse.communication.ontology.SA_search_golf_courses;
import golfCourse.communication.ontology.TA_translate_to_OTAGolfCourseSearchRS;
import golfCourse.communication.services.RequestHandler;
import golfCourse.dao.GolfCourseDAO;
import golfCourse.exceptions.DAOException;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

import com.hp.hpl.jena.ontology.OntModel;


public class FindGolfCoursesHandler extends RequestHandler {

	private static Logger logger = Logger.getMyLogger(FindGolfCoursesHandler.class.getName());
	
	public final static int DEFAULT_MAX_RESULTS = 100;
	
	private GolfCourseDAO daoGolfCourses;

	private OntModel mDomain;
	private OntModel mGolfCoursesDB;
	
	protected SearchAgent myAgent;
	
	public FindGolfCoursesHandler(Agent a, DataStore ds, Object requestKey,
			Object actKey, Object notificationKey) {
		super(a, ds, requestKey, actKey, notificationKey);
		myAgent = (SearchAgent) a;
		
		mDomain = myAgent.getDomainModel();
		mGolfCoursesDB = myAgent.getGolfCoursesDB();
		daoGolfCourses = new GolfCourseDAO(mDomain, mGolfCoursesDB);
	}

	protected final ACLMessage prepareResponse(ACLMessage request, Action act) throws FailureException {
		logger.info(myAgent.getLocalName() + " - search golf courses action");	
		try {
				logger.info(myAgent.getLocalName() + ": receive conditions");
				SA_search_golf_courses action = (SA_search_golf_courses) act.getAction();
					
				ACLMessage response = null;	
				Conditions conditions = action.getCoursesConditions();
				
				try {
				OWLData owlData = daoGolfCourses.findGolfCourses(conditions);
				
				logger.info(myAgent.getLocalName() + " executed query");
				logger.info(myAgent.getLocalName() + " found " + owlData.getWithRootURI().size() + " golf courses");

				if (action instanceof SA_return_golf_courses)
				{
					response = prepareMessageToMA(request, owlData, action.getCustomerMessageID());
				}
				else
				{
					response = prepareMessageToTA(request, owlData, action.getCustomerMessageID());				
				}
				}
				catch(DAOException e)
				{
					// Message to MA about error during search golfCourses
					if (action instanceof SA_return_golf_courses)
					{
						myAgent.addBehaviour(new SendErrorToExternalBehaviour(e.getMessage(), myAgent, action.getCustomerMessageID()));
					}
					else
					{
						myAgent.addBehaviour(new SendErrorToOTABehaviour(e.getMessage(), myAgent, action.getCustomerMessageID()));			
					}
				}
				return response;
				
			} catch (Codec.CodecException e) {
				logger.info(myAgent.getLocalName() + " - cannot realize request " + e.getMessage());
				
				myAgent.addBehaviour(new SendSystemErrorBehaviour(myAgent.getLocalName() + " - cannot realize request " + e.getMessage(), myAgent));
				throw new FailureException(e.getMessage());
			}
			catch (OntologyException e) {
				logger.info(myAgent.getLocalName() + " - cannot realize request " + e.getMessage());
				myAgent.addBehaviour(new SendSystemErrorBehaviour(myAgent.getLocalName() + " - cannot realize request " + e.getMessage(), myAgent));
				throw new FailureException(e.getMessage());
			}
	}
	
	private ACLMessage prepareMessageToMA(ACLMessage request, OWLData owlData, String customerID) throws Codec.CodecException, OntologyException
	{
		logger.info("Search agent - prepare message to MA");
		ACLMessage response = request.createReply();
		MA_send_ontologies responseAction = new MA_send_ontologies();
		responseAction.setCustomerMessageID(customerID);
		responseAction.setOntologies_(owlData);
		//ACLMessage response = new ACLMessage(ACLMessage.INFORM);
		myAgent.getContentManager().fillContent(response, new Action(myAgent.getAID(), responseAction));
	
		logger.info(myAgent.getLocalName() + " created response message with ontologies - quey results");

		return response;
	}

	private ACLMessage prepareMessageToTA(ACLMessage request, OWLData owlData, String customerID) throws Codec.CodecException, OntologyException
	{
		logger.info("Search agent - prepare message to TA");
		ACLMessage response = request.createReply();
		
		// action instanceof SA_send_golf_courses_to_TA
		TA_translate_to_OTAGolfCourseSearchRS responseAction = new TA_translate_to_OTAGolfCourseSearchRS();
	
		responseAction.setCustomerMessageID(customerID);
	
		responseAction.setGolfCourseOntologies(owlData);
	
		myAgent.getContentManager().fillContent(response, new Action(myAgent.getAID(), responseAction));
	
		logger.info(myAgent.getLocalName() + " created response message with ontologies - quey results");

		return response;
	}
}
