package golfCourse.dao;

import golfCourse.agents.managerAgent.behaviours.CustomerResponseBehaviour;
import golfCourse.communication.ontology.Condition;
import golfCourse.communication.ontology.Conditions;
import golfCourse.communication.ontology.OWLData;
import golfCourse.exceptions.DAOException;
import golfCourse.ontologies.jastor.contacts.Contacts;
import golfCourse.ontologies.jastor.contacts.ContactsFactory;
import golfCourse.ontologies.jastor.golfCourse.GolfCourse;
import golfCourse.ontologies.jastor.golfCourse.GolfCourseFactory;
import golfCourse.ontologies.jastor.location.AddressRecord;
import golfCourse.ontologies.jastor.location.AddressRecordFactory;
import golfCourse.ontology.semantic.Memory;
import golfCourse.properties.ConditionOperation;
import golfCourse.translations.ConditionsVocabulary;

import jade.util.Logger;

import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.datatypes.RDFDatatype;
import com.hp.hpl.jena.datatypes.TypeMapper;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.core.ElementFilter;
import com.hp.hpl.jena.query.core.ElementGroup;
import com.hp.hpl.jena.query.core.ElementTriplePattern;
import com.hp.hpl.jena.query.expr.E_GreaterThan;
import com.hp.hpl.jena.query.expr.E_GreaterThanOrEqual;
import com.hp.hpl.jena.query.expr.E_LessThan;
import com.hp.hpl.jena.query.expr.E_NotEquals;
import com.hp.hpl.jena.query.expr.Expr;
import com.hp.hpl.jena.query.expr.NodeValue;
import com.hp.hpl.jena.query.expr.NodeVar;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.vocabulary.RDF;
import com.ibm.adtech.jastor.JastorException;

public class GolfCourseDAO {

	// Model with domain ontology.
	private OntModel mDomain;
	// Model with restaurants descriptions.
	private OntModel mGolfCoursesDB;
	
	private final static String golfCourseName = "GolfCourse";
	private final static String addressName = "Address";
	private final static String contactsName = "Contacts";
	
	private static Logger logger = Logger.getMyLogger(GolfCourseDAO.class.getName());

	public GolfCourseDAO(OntModel mDomain, OntModel mGolfCoursesDB) {
		this.mDomain = mDomain;
		this.mGolfCoursesDB = mGolfCoursesDB;
	}
	
	public OWLData findGolfCourses(Conditions conditions) throws DAOException
	{
		Query query = buildQuery(conditions);
		return executeQuery(query, conditions);
	}
	
	private Query buildQuery(Conditions conditions) throws DAOException
	{		
		String baseURI = mDomain.getNsPrefixURI("base");
		Node varGolfCourse = Node.createVariable(golfCourseName);
		Node varAddress = Node.createVariable(addressName);
		Node varContact = Node.createVariable(contactsName);
		
		Query query = QueryFactory.make();
		query.setQueryType(Query.QueryTypeSelect);
		query.addResultVar(varGolfCourse);
		query.addResultVar(varAddress);
		query.addResultVar(varContact);
		
		query.getPrefixMapping().setNsPrefix("rdf", RDF.getURI());
		query.getPrefixMapping().setNsPrefix("golfCourse", baseURI);
		
		ElementGroup elementGroup = new ElementGroup();
		
		Triple golfCourseTriple = new Triple(varGolfCourse, RDF.type.asNode(), GolfCourse.TYPE.asNode());
		
		elementGroup.addElement(new ElementTriplePattern(golfCourseTriple));
		
		Triple addressTriple = new Triple(varAddress, RDF.type.asNode(), AddressRecord.TYPE.asNode());
		
		elementGroup.addElement(new ElementTriplePattern(addressTriple));
		
		Triple contactTriple = new Triple(varContact, RDF.type.asNode(), Contacts.TYPE.asNode());
		
		elementGroup.addElement(new ElementTriplePattern(contactTriple));
		
		OntProperty property2 = mDomain.getOntProperty(baseURI + "address");
		
		Triple a3 = new Triple(varGolfCourse, property2.asNode(), varAddress);
		elementGroup.addElement(new ElementTriplePattern(a3)); 
		
		OntProperty  property3 = mDomain.getOntProperty(baseURI + "contactInfo");
		Triple a5 = new Triple(varGolfCourse, property3.asNode(), varContact);
		elementGroup.addElement(new ElementTriplePattern(a5));
		
		for(jade.util.leap.Iterator iter = conditions.getAllConditions(); iter.hasNext();)
		{
			Condition condition = (Condition)iter.next();
			if(condition.getRequired_())
				generate(baseURI, condition, varGolfCourse, elementGroup);
		}
		
		query.setQueryElement(elementGroup);
		
		logger.info("\n\n Query: \n\n" + query.toString() + "\n\n");
		return query;
	}
	
	private void generate(String baseURI, Condition trait, Node varGolfCourse, ElementGroup elementGroup)throws DAOException
	{
		OntProperty property = mDomain.getOntProperty(baseURI+trait.getName_());
		if(property==null)
			return;
		
		Node object;
		if(property.isDatatypeProperty()) {
			String rangeURI = property.getRange().getURI();
			RDFDatatype type = TypeMapper.getInstance().getTypeByName(rangeURI);
			object = Node.createLiteral(trait.getValueString(),"", type);
		}
		else if (property.canAs(ObjectProperty.class))
		{
			object = Node.createURI(trait.getValueString());
		}
		else throw new DAOException("Urecognized type of property");
		
		if(trait.getOperation_().equals(ConditionOperation.GREATER_THAN_OR_EQUAL) ||
				trait.getOperation_().equals(ConditionOperation.GREATER_THAN) ||
				trait.getOperation_().equals(ConditionOperation.LESS_THAN) ||
				trait.getOperation_().equals(ConditionOperation.NOT_EQUAL))
		{
			Expr arg2 = (Expr)NodeValue.makeNode(object);//(Expr)NodeValue.makeInteger(trait.getValue());
			
			NodeVar nodeVar = new NodeVar(trait.getName_());
			Expr arg1 = (Expr) nodeVar;
			
			Expr c = generateExpr(arg1, arg2, trait.getOperation_());//new E_LessThan(arg1, arg2);
			
			ElementFilter filter = new ElementFilter(c);
			
			Node traitNode = Node.createVariable(trait.getName_());
			Triple triple = new Triple(varGolfCourse, property.asNode(), traitNode);
			elementGroup.addElement(new ElementTriplePattern(triple)); 	
			
			elementGroup.addElement(filter);
		}
		else
		{
			Triple triple = new Triple(varGolfCourse, property.asNode(), object);
			elementGroup.addElement(new ElementTriplePattern(triple)); 
		}
	}
	
	private static Expr generateExpr(Expr arg1, Expr arg2, String operation)
	{
		if(operation.equals(ConditionOperation.LESS_THAN))
			return new E_LessThan(arg1, arg2);
		
		if(operation.equals(ConditionOperation.GREATER_THAN))
			return new E_GreaterThan(arg1, arg2);
		
		if(operation.equals(ConditionOperation.NOT_EQUAL))
			return new E_NotEquals(arg1, arg2);
		
		if(operation.equals(ConditionOperation.GREATER_THAN_OR_EQUAL))
			return new E_GreaterThanOrEqual(arg1, arg2);
		
		return null;
	}
	
	private OWLData executeQuery(Query query, Conditions conditions) throws DAOException
	{
		QueryExecution qexec = null;
		qexec = QueryExecutionFactory.create(query, mGolfCoursesDB);

		ResultSet results = qexec.execSelect();
		
		return getSearchResults(results, conditions);
	} 
	
	private OWLData getSearchResults(ResultSet results, Conditions conditions) throws DAOException
	{
		List golfCourses = new ArrayList();
		
		OWLData courses = new OWLData();
		Memory memory = new Memory();
		
		int i = 0;
		
		try {
		while(results.hasNext())
		{
			i++;
			QuerySolution soln = results.nextSolution();
			
			Resource res = soln.getResource(golfCourseName);
			Resource add = soln.getResource(addressName);
			Resource contact = soln.getResource(contactsName);
			
			//courses.addWithRootURI(res.toString());
			
			GolfCourse golf = GolfCourseFactory.createGolfCourse(res, mGolfCoursesDB);
			AddressRecord address = AddressRecordFactory.createAddressRecord(add, mGolfCoursesDB);
			Contacts contacts = ContactsFactory.createContacts(contact, mGolfCoursesDB);
			
			golfCourses.add(golf);
			
			memory.add(contacts);
			memory.add(golf);
			memory.add(address);
		}
		
		// sort list golfCourses
		List sortGolfCourses = sortGolfCourses(golfCourses, conditions);
		
		for(int index = 0; index < sortGolfCourses.size(); index++)
		{
			GolfCourse golfCourse = (GolfCourse)sortGolfCourses.get(index);
			courses.addWithRootURI(golfCourse.uri());
		}
		
		courses.setDataModel(memory.dump());
		}
		catch(JastorException e)
		{
			throw new DAOException(e.getMessage());
		}
		
		return courses;
	}
	
	private List sortGolfCourses(List golfCourses, Conditions conditions) throws DAOException
	{
		logger.info("List golfCourses: " + golfCourses.size());	
		List notRequiredConditions = getNotRequiredConditions(conditions);
		
		// no sort
		if(notRequiredConditions.size() == 0)
			return golfCourses;
		
		List courses = new ArrayList();
		
		for(int i = 0; i < golfCourses.size(); i++)
		{
			courses.add(new GolfCourseWithConditions((GolfCourse)golfCourses.get(i)));
		}
		
		for(int i = 0; i < notRequiredConditions.size(); i++)
		{
			Condition condition = (Condition)notRequiredConditions.get(i);
			
			for(int j = 0; j < courses.size(); j++)
			{
				GolfCourseWithConditions golfCourse = (GolfCourseWithConditions)courses.get(j);
				
				if(hasCondition(golfCourse.golfCourse, condition))
					golfCourse.count++;
			}
		}
		
		Collections.sort(courses);		
		return getGolfCourses(courses);
	}

	private List getGolfCourses(List golfCoursesWithConditions)
	{
		List list = new ArrayList();
		
		for(int i = golfCoursesWithConditions.size() - 1; i >= 0; i--)
		{
			GolfCourseWithConditions elem = (GolfCourseWithConditions)golfCoursesWithConditions.get(i);
			list.add(elem.golfCourse);
		}
		return list;
	}
	
	private String getCondition(GolfCourse golfCourse, String conditionName) throws JastorException
	{
		if(ConditionsVocabulary.ONT_ADA_CHALLENGED.equals(conditionName))
			return "";
		
		if(ConditionsVocabulary.ONT_CADDIES_AVAILABLE.equals(conditionName))
			return golfCourse.getAvailCaddy().toString();
		
		if(ConditionsVocabulary.ONT_GRASS_TYPE.equals(conditionName))
			return golfCourse.getGrassType();
		
		if(ConditionsVocabulary.ONT_METAL_SPIKES.equals(conditionName))
			return golfCourse.getMetalSpikes().toString();
		
		if(ConditionsVocabulary.ONT_PERSONAL_CARTS.equals(conditionName))
			return golfCourse.getPermCart().toString();
		
		if(ConditionsVocabulary.ONT_SINGLESS_CONFIRMED.equals(conditionName))
			return golfCourse.getSinglesConfirmed().toString();
		
		if(ConditionsVocabulary.ONT_SLOPE.equals(conditionName))
			return golfCourse.getSlope().toString();
		
		if(ConditionsVocabulary.ONT_YARDAGE.equals(conditionName))
			return golfCourse.getYardage().toString();
		
		return "";
	}
	
	private boolean hasCondition(GolfCourse golfCourse, Condition condition) throws DAOException
	{
		try {
			String conditionName = condition.getName_();
		
			if(ConditionsVocabulary.ONT_ARCHITECT.equals(conditionName))
			{
				for(Iterator iter = golfCourse.getArchitect(); iter.hasNext();)
				{
					String architect = (String)iter.next();
					if(condition.getValueString().equals(architect))
						return true;
				}
				return false;
			}
		
			String value = getCondition(golfCourse, conditionName);
		
			if(value.equals(condition.getValueString()))
				return true;
		}catch(JastorException e)
		{
			throw new DAOException("Exception during sort: " + e.getMessage());
		}
		return false;
	}
	
	private List getNotRequiredConditions(Conditions conditions)
	{
		List list = new ArrayList();
		
		for(Iterator iter = conditions.getAllConditions(); iter.hasNext();)
		{
			Condition condition = (Condition)iter.next();
			if(!condition.getRequired_())
				list.add(condition);
		}
		return list;
	}
	
	public void loadData(OWLData data)throws DAOException
	{
		Memory memory = new Memory();
		memory.add(data);
		OntModel model = memory.getModel();
		
		for(Iterator iter = data.getWithRootURI().iterator(); iter.hasNext();) {
			String uri = (String)iter.next();
	
			GolfCourse golfCourse = null;
			AddressRecord address = null;
			Contacts contacts = null;
			
			try {
				golfCourse = GolfCourseFactory.createGolfCourse(uri, model);
				if(golfCourse.getAddress() != null)
				{
					String addressURI = golfCourse.getAddress().uri();
					address = AddressRecordFactory.createAddressRecord(addressURI, model);
				}
			
				if(golfCourse.getContactInfo() != null)
				{
					String contactURI = golfCourse.getContactInfo().uri();
					contacts = ContactsFactory.createContacts(contactURI, model);
				}
			}catch(JastorException e)
			{
				throw new DAOException(e.getMessage());
			}
			
			loadData(golfCourse, address, contacts);
		}
	}
	
	private void loadData(GolfCourse golfCourse, AddressRecord address, Contacts contacts) throws DAOException
	{
		GolfCourse newGolfCourse;
		AddressRecord newAddress;
		Contacts newContact;
		
		try {
			mGolfCoursesDB.begin();
			
			Resource golfCourseResource = mGolfCoursesDB.createResource(GolfCourse.TYPE.getNameSpace() + golfCourse.getId());
			
			Resource addressResource = mGolfCoursesDB.createResource(AddressRecord.TYPE.getNameSpace() + golfCourse.getId()+"_address");
			Resource contactResource = mGolfCoursesDB.createResource(Contacts.TYPE.getNameSpace() + golfCourse.getId()+"_contact");
			
			if (mGolfCoursesDB.getBaseModel().contains(golfCourseResource, RDF.type, GolfCourse.TYPE))
			{
				logger.info("update -- " + golfCourse.uri());
				newGolfCourse = GolfCourseFactory.getGolfCourse(golfCourseResource, mGolfCoursesDB);
			}
			else
			{
				logger.info("insert -- " + golfCourse.uri());
				newGolfCourse = GolfCourseFactory.createGolfCourse(golfCourseResource, mGolfCoursesDB);
			}
			
			DAOTools.copyProperties(newGolfCourse, golfCourse);
			
			if(mGolfCoursesDB.getBaseModel().contains(addressResource, RDF.type, AddressRecord.TYPE))
			{
				logger.info("update address -- " + address.uri());
				newAddress = AddressRecordFactory.getAddressRecord(addressResource, mGolfCoursesDB);
			}
			else
			{
				logger.info("insert address -- " + address.uri());
				newAddress = AddressRecordFactory.createAddressRecord(addressResource, mGolfCoursesDB);
			}
			
			DAOTools.copyProperties(newAddress, address);
			newGolfCourse.setAddress(newAddress);
			
			if(mGolfCoursesDB.getBaseModel().contains(contactResource, RDF.type, Contacts.TYPE))
			{
				logger.info("update contacts -- " + contacts.uri());
				newContact = ContactsFactory.getContacts(contactResource, mGolfCoursesDB);
			}
			else
			{
				logger.info("insert contacts -- " + contacts.uri());
				newContact = ContactsFactory.createContacts(contactResource, mGolfCoursesDB);
			}
			
			DAOTools.copyProperties(newContact, contacts);
			newGolfCourse.setContactInfo(newContact);
			mGolfCoursesDB.commit();
		} catch (JastorException e) {
			mGolfCoursesDB.abort();
			throw new DAOException(e.getMessage());
		}
	}
	
	/**
	 * Remove all data from database
	 */
	public void removeAllData() {
		mGolfCoursesDB.begin();
		mGolfCoursesDB.removeAll();
		mGolfCoursesDB.commit();
	}
}
