package golfCourse.translations.fromOTA;

import golfCourse.communication.ontology.OWLData;
import golfCourse.exceptions.TranslateException;
import golfCourse.ontologies.jastor.contacts.Contacts;
import golfCourse.ontologies.jastor.golfCourse.GolfCourse;
import golfCourse.ontologies.jastor.golfCourse.GolfCourseFactory;
import golfCourse.ontologies.jastor.location.AddressRecord;
import golfCourse.ontology.semantic.Memory;
import golfCourse.properties.GolfCourseSystemProperties;
import golfCourse.translations.ConditionsVocabulary;
import golfCourse.translations.castor.GolfCourses;
import golfCourse.translations.castor.OTA_GolfCourseSearchRS;
import golfCourse.translations.castor.Trait;
import golfCourse.translations.castor.Traits;

import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Resource;
import com.ibm.adtech.jastor.JastorException;


// W przypadku gdy pytamy sie innych zrodel o pola golfowe i gdy musimy wyniki prztumaczyc
// na ontologie zeby potem zapisac do bazy danych

public class GolfCourseSearchRSTranslator {

	private OntModel golfCourseModel;
	private AddressTranslator addressTranslator;
	private ContactTranslator contactTranslator;
	
	public GolfCourseSearchRSTranslator(OntModel golfCourseModel, OntModel addressModel, OntModel contactModel) {
		this.golfCourseModel = golfCourseModel;
		this.addressTranslator = new AddressTranslator(addressModel);
		this.contactTranslator = new ContactTranslator(contactModel);
	}
	
	/**
	 * Returns list of instances ontologies with informacion about Golf Course
	 * instances of ontologies are in GolfCourse class generated from jastor
	 * @param text
	 * @return list of instances of ontologies
	 */
	public OWLData translateGolfCoursesToOntologies(String otaString) throws TranslateException
	{
		Reader reader = new StringReader(otaString);
		
		try {
			OTA_GolfCourseSearchRS ota = (OTA_GolfCourseSearchRS)OTA_GolfCourseSearchRS.unmarshal(reader);
			OWLData ontologies = translateGolfCoursesToOntologies(ota.getOTA_GolfCourseSearchRSSequence().getGolfCourses());
			return ontologies;
		}catch(MarshalException e)
		{
			throw new TranslateException(" marshal: " + e.toString());
		}
		catch(ValidationException e2)
		{
			throw new TranslateException(" validation: " + e2.toString());
		}
		catch(JastorException e3)
		{
			throw new TranslateException(" jastor: " + e3.toString());
		}
	}
	
	private OWLData translateGolfCoursesToOntologies(GolfCourses golfCourses) throws JastorException
	{
		OWLData ontologies = new OWLData();
		Memory memory = new Memory();
		
		for(int i = 0; i < golfCourses.getGolfCourseCount(); i++)
		{
			golfCourse.translations.castor.GolfCourse otaGolfCourse = golfCourses.getGolfCourse(i);
			GolfCourse golfCourse = translateGolfCourse(otaGolfCourse);
			
			ontologies.addWithRootURI(golfCourse.uri());
			
			AddressRecord address = addressTranslator.translate(otaGolfCourse.getAddress(), golfCourse.getId());
			golfCourse.setAddress(address);
			
			Contacts contacts = contactTranslator.translate(otaGolfCourse.getPhone(), golfCourse.getId());
			golfCourse.setContactInfo(contacts);
			
			memory.add(address);
			memory.add(contacts);
			memory.add(golfCourse);
		}
		ontologies.setDataModel(memory.dump());
		return ontologies;
	}
	
	private GolfCourse translateGolfCourse(golfCourse.translations.castor.GolfCourse otaGolfCourse) throws JastorException
	{
		Resource r = golfCourseModel.createResource(GolfCourse.TYPE.getNameSpace() + otaGolfCourse.getID());
		GolfCourse ontGolfCourse = GolfCourseFactory.createGolfCourse(r, golfCourseModel);
		
		ontGolfCourse.setName(otaGolfCourse.getName());
		ontGolfCourse.setId(otaGolfCourse.getID());
		ontGolfCourse = translateTraits(ontGolfCourse, otaGolfCourse.getTraits());
		
		// TODO add translation of address etc.
		return ontGolfCourse;
	}
	
	
	private GolfCourse translateTraits(GolfCourse golfCourse, Traits traits) throws JastorException
	{
		for(int i = 0; i < traits.getTraitCount(); i++)
		{
			golfCourse = translateTrait(golfCourse, traits.getTrait(i));
		}
		return golfCourse;
	}
	
	private GolfCourse translateTrait(GolfCourse golfCourse, Trait trait) throws JastorException
	{
		String name = trait.getName();
		
		if(name.equals(ConditionsVocabulary.OTA_ARCHITECT))
			golfCourse.addArchitect(trait.getValue());
	
		if(name.equals(ConditionsVocabulary.OTA_CADDIES_AVAILABLE))
			golfCourse.setAvailCaddy(Boolean.valueOf(trait.getValue()));
		
		if(name.equals(ConditionsVocabulary.OTA_METAL_SPIKES))
			golfCourse.setMetalSpikes(Boolean.valueOf(trait.getValue()));
		
		if(name.equals(ConditionsVocabulary.OTA_PERSONAL_CARTS))
			golfCourse.setPermCart(Boolean.valueOf(trait.getValue()));
		
		if(name.equals(ConditionsVocabulary.OTA_SINGLESS_CONFIRMED))
			golfCourse.setSinglesConfirmed(Boolean.valueOf(trait.getValue()));
		
		if(name.equals(ConditionsVocabulary.OTA_YARDAGE))
			golfCourse.setYardage(Float.valueOf(trait.getValue()));
		
	//	if(name.equals(GolfCourseVocabulary.OTA_SLOPE))
	//		golfCourse.setSlope(Integer.getInteger(trait.getValue()));
		
		
		/*if(name.equals(GolfCourseVocabulary.OTA_ADA_CHALLENGED))
		{
			golfCourse.set
		}*/
		
		return golfCourse;
	}
	
}
